/*
  Copyright (C) 2001 Kimmo Pekkola

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/*
  $Header: /home/cvsroot/Rainmeter/Plugin/MeasureNet.cpp,v 1.10 2004/06/05 10:55:54 rainy Exp $

  $Log: MeasureNet.cpp,v $
  Revision 1.10  2004/06/05 10:55:54  rainy
  Too much changes to be listed in here...

  Revision 1.9  2003/12/05 15:50:09  Rainy
  Multi-instance changes.

  Revision 1.8  2003/02/10 18:13:34  rainy
  Changed the way stats are gathered.

  Revision 1.7  2002/12/23 14:26:07  rainy
  Added cumulative statistics measuring.

  Revision 1.6  2002/07/01 15:34:23  rainy
  Now it is possible to select the NIC.

  Revision 1.5  2002/04/26 18:24:16  rainy
  Modified the Update method to support disabled measures.

  Revision 1.4  2002/04/01 15:38:25  rainy
  Some on the implementation from NetIn/Out is moved here.
  Added TrafficAction and TrafficValue.

  Revision 1.3  2002/03/31 09:58:54  rainy
  Added some comments

  Revision 1.2  2001/08/12 15:45:34  Rainy
  Changed UpdateTable() to be a class method.

  Revision 1.1.1.1  2001/08/11 10:58:19  Rainy
  Added to CVS.

*/

#include "MeasureNet.h"
#include "Rainmeter.h"

MIB_IFTABLE* CMeasureNet::c_Table = NULL;
int CMeasureNet::c_NumOfTables = 0;
std::vector<LARGE_INTEGER> CMeasureNet::c_StatValues;
std::vector<DWORD> CMeasureNet::c_OldStatValues;

/*
** CMeasureNet
**
** The constructor. This is the base class for the net-meters. 
**
*/
CMeasureNet::CMeasureNet() : CMeasure()
{
	m_TrafficValue = 0;
	m_CurrentTraffic = 0;
	m_Interface = 0;
	m_Cumulative = false;
}

/*
** ~CMeasureNet
**
** The destructor
**
*/
CMeasureNet::~CMeasureNet()
{
	delete [] c_Table;
	c_Table = NULL;
}

/*
** Update
**
** Checks if Action should be executed.
**
*/
bool CMeasureNet::Update(CMeterWindow* meterWindow)
{
	if (!CMeasure::PreUpdate(meterWindow)) return false;

	if (meterWindow)
	{
		if (!m_TrafficAction.empty())
		{
			if (m_CurrentTraffic > m_TrafficValue)
			{
				m_CurrentTraffic = 0;
				meterWindow->ExecuteCommand(m_TrafficAction.c_str());
			}

			m_CurrentTraffic += m_Value;
		}
	}

	return PostUpdate(meterWindow);
}

/*
** UpdateIFTable
**
** Reads the tables for all net interfaces
**
*/
void CMeasureNet::UpdateIFTable()
{
	if(c_Table == NULL)
	{
		// Gotta reserve few bytes for the tables
		DWORD value;
		if(GetNumberOfInterfaces(&value) == NO_ERROR)
		{
			c_NumOfTables = value;

			if(c_NumOfTables > 0)
			{
				DWORD size = sizeof(MIB_IFTABLE) + sizeof(MIB_IFROW) * c_NumOfTables;
				c_Table = (MIB_IFTABLE*)new char[size];
			}
		}
	}

	if(c_Table)
	{
		DWORD size = sizeof(MIB_IFTABLE) + sizeof(MIB_IFROW) * c_NumOfTables;
		if(GetIfTable(c_Table, &size, false) != NO_ERROR)
		{
			delete [] c_Table;
			c_Table = (MIB_IFTABLE*)new char[size];
			if(GetIfTable(c_Table, &size, false) != NO_ERROR)
			{
				// Something's wrong. Unable to get the table.
				delete [] c_Table;
				c_Table = NULL;
			}
		}
	}
}

/*
** GetNetOctets
**
** Reads the amount of octets. This is the same for in, out and total.
** the net-parameter informs which inherited class called this method.
**
*/
DWORD CMeasureNet::GetNetOctets(NET net)
{
	DWORD value = 0;

	if (m_Interface == 0)
	{
		// Get all interfaces
		for(int i = 0; i < c_NumOfTables; i++)
		{
			// Ignore the loopback
			if(strcmp((char*)c_Table->table[i].bDescr, "MS TCP Loopback interface") != 0)
			{
				switch (net)
				{
				case NET_IN:
					value += c_Table->table[i].dwInOctets;
					break;

				case NET_OUT:
					value += c_Table->table[i].dwOutOctets;
					break;

				case NET_TOTAL:
					value += c_Table->table[i].dwInOctets;
					value += c_Table->table[i].dwOutOctets;
					break;
				}
			}
		}
	}
	else
	{
		// Get the selected interface
		if (m_Interface <= c_NumOfTables)
		{
			switch (net)
			{
			case NET_IN:
				value += c_Table->table[m_Interface - 1].dwInOctets;
				break;

			case NET_OUT:
				value += c_Table->table[m_Interface - 1].dwOutOctets;
				break;

			case NET_TOTAL:
				value += c_Table->table[m_Interface - 1].dwInOctets;
				value += c_Table->table[m_Interface - 1].dwOutOctets;
				break;
			}
		}
	}

	return value;
}

/*
** GetNetStatsValue
**
** Returns the stats value of the interface
**
*/
LARGE_INTEGER CMeasureNet::GetNetStatsValue(NET net)
{
	LARGE_INTEGER value;
	value.QuadPart = 0;

	if (m_Interface == 0)
	{
		// Get all interfaces
		for(int i = 0; i < c_StatValues.size() / 2; i++)
		{
			switch (net)
			{
			case NET_IN:
				value.QuadPart += c_StatValues[i * 2 + 0].QuadPart;
				break;

			case NET_OUT:
				value.QuadPart += c_StatValues[i * 2 + 1].QuadPart;
				break;

			case NET_TOTAL:
				value.QuadPart += c_StatValues[i * 2 + 0].QuadPart;
				value.QuadPart += c_StatValues[i * 2 + 1].QuadPart;
				break;
			}
		}
	}
	else
	{
		// Get the selected interface
		if (m_Interface <= c_StatValues.size() / 2)
		{
			switch (net)
			{
			case NET_IN:
				value.QuadPart += c_StatValues[m_Interface * 2 + 0].QuadPart;
				break;

			case NET_OUT:
				value.QuadPart += c_StatValues[m_Interface * 2 + 1].QuadPart;
				break;

			case NET_TOTAL:
				value.QuadPart += c_StatValues[m_Interface * 2 + 0].QuadPart;
				value.QuadPart += c_StatValues[m_Interface * 2 + 1].QuadPart;
				break;
			}
		}
	}

	return value;
}

/*
** ReadConfig
**
** Reads the measure specific configs. This is the same for in, out and total.
** the net-parameter informs which inherited class called this method.
**
*/
void CMeasureNet::ReadConfig(CConfigParser& parser, const char* section, NET net)
{
	int value;
	const char* netName = NULL;

	if (net == NET_IN)
	{
		netName = "NetInSpeed";
		value = CRainmeter::GetGlobalConfig().netInSpeed;
	}
	else if (net == NET_OUT)
	{
		netName = "NetOutSpeed";
		value = CRainmeter::GetGlobalConfig().netOutSpeed;
	}
	else
	{
		netName = "NetTotalSpeed";
		value = CRainmeter::GetGlobalConfig().netInSpeed + CRainmeter::GetGlobalConfig().netOutSpeed;
	}

	value = parser.ReadInt(section, netName, value);

	m_Interface = parser.ReadInt(section, "Interface", 0);
	m_Cumulative = 0!=parser.ReadInt(section, "Cumulative", 0);

	m_TrafficValue = parser.ReadFloat(section, "TrafficValue", 0.0);
	m_TrafficAction = parser.ReadString(section, "TrafficAction", "");

	if(value == 0)
	{
		m_MaxValue = 1;
		m_LogMaxValue = true;
	}
	else
	{
		m_MaxValue = value / 8;
	}
}

/*
** UpdateStats
**
** Updates the statistics.
**
*/
void CMeasureNet::UpdateStats()
{
	if (c_Table)
	{
		// Fill the vectors
		while (c_StatValues.size() < c_Table->dwNumEntries * 2)
		{
			LARGE_INTEGER value;
			value.QuadPart = 0;
			c_StatValues.push_back(value);
		}

		while (c_OldStatValues.size() < c_Table->dwNumEntries * 2)
		{
			c_OldStatValues.push_back(0);
		}

		for (int i = 0; i < c_Table->dwNumEntries; i++)
		{
			DWORD in, out;

			in = c_Table->table[i].dwInOctets;
			out = c_Table->table[i].dwOutOctets;

			if (c_OldStatValues[i * 2 + 0] != 0 && c_OldStatValues[i * 2 + 1] != 0)
			{
				c_StatValues[i * 2 + 0].QuadPart += in - c_OldStatValues[i * 2 + 0];
				c_StatValues[i * 2 + 1].QuadPart += out - c_OldStatValues[i * 2 + 1];
			}

			c_OldStatValues[i * 2 + 0] = in;
			c_OldStatValues[i * 2 + 1] = out;
		}
	}
}

/*
** ResetStats
**
** Resets the statistics.
**
*/
void CMeasureNet::ResetStats()
{
	if (c_Table)
	{
		for (int i = 0; i < c_StatValues.size(); i++)
		{
			LARGE_INTEGER value;
			value.QuadPart = 0;
			c_StatValues[i] = value;
		}
	}
}

/*
** ReadStats
**
** Reads statistics.
**
*/
void CMeasureNet::ReadStats(const std::string& iniFile)
{
	char buffer[256];
	int count = GetPrivateProfileInt("Statistics", "NetStatsCount", 0, iniFile.c_str());

	c_StatValues.clear();

	for (int i = 0; i < count; i++)
	{
		LARGE_INTEGER value;

		sprintf(buffer, "NetStatsInHigh%i", i + 1);
		value.HighPart = (DWORD)GetPrivateProfileInt("Statistics", buffer, 0, iniFile.c_str());
		sprintf(buffer, "NetStatsInLow%i", i + 1);
		value.LowPart = (DWORD)GetPrivateProfileInt("Statistics", buffer, 0, iniFile.c_str());
		c_StatValues.push_back(value);

		sprintf(buffer, "NetStatsOutHigh%i", i + 1);
		value.HighPart = (DWORD)GetPrivateProfileInt("Statistics", buffer, 0, iniFile.c_str());
		sprintf(buffer, "NetStatsOutLow%i", i + 1);
		value.LowPart = (DWORD)GetPrivateProfileInt("Statistics", buffer, 0, iniFile.c_str());
		c_StatValues.push_back(value);
	}
}

/*
** WriteStats
**
** Writes statistics.
**
*/
void CMeasureNet::WriteStats(const std::string& iniFile)
{
	char buffer[256];
	char buffer2[256];
	
	sprintf(buffer, "%i", c_StatValues.size() / 2);
	WritePrivateProfileString("Statistics", "NetStatsCount", buffer, iniFile.c_str());

	for (int i = 0; i < c_StatValues.size() / 2; i++)
	{
		sprintf(buffer2, "NetStatsInHigh%i", i + 1);
		sprintf(buffer, "%u", c_StatValues[i * 2].HighPart);
		WritePrivateProfileString("Statistics", buffer2, buffer, iniFile.c_str());

		sprintf(buffer2, "NetStatsInLow%i", i + 1);
		sprintf(buffer, "%u", c_StatValues[i * 2].LowPart);
		WritePrivateProfileString("Statistics", buffer2, buffer, iniFile.c_str());

		sprintf(buffer2, "NetStatsOutHigh%i", i + 1);
		sprintf(buffer, "%u", c_StatValues[i * 2 + 1].HighPart);
		WritePrivateProfileString("Statistics", buffer2, buffer, iniFile.c_str());

		sprintf(buffer2, "NetStatsOutLow%i", i + 1);
		sprintf(buffer, "%u", c_StatValues[i * 2 + 1].LowPart);
		WritePrivateProfileString("Statistics", buffer2, buffer, iniFile.c_str());
	}
}

